@@ -3,7 +3,7 @@ package JavaScript::Beautifier;
 use warnings;
 use strict;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 our $AUTHORITY = 'cpan:FAYLAND';
 
 use base 'Exporter';
@@ -19,19 +19,20 @@ my @punct      = split(' ', '+ - * / % & ++ -- = += -= *= /= %= == === != !== >
 # words which should always start on new line.
 my @line_starter = split(',', 'continue,try,throw,return,var,if,switch,case,default,for,while,break,function');
 
-my ( $indent_level, $indent_size, $indent_character );
+my ( $opt_indent_level, $opt_indent_size, $opt_indent_character, $opt_preserve_newlines );
 
 sub js_beautify {
     my ( $js_source_code, $opts ) = @_;
     
-    $indent_size = $opts->{indent_size} || 4;
-    $indent_character = $opts->{indent_character} || ' ';
-    $indent_level = $opts->{indent_level} ||= 0;
+    $opt_indent_size = $opts->{indent_size} || 4;
+    $opt_indent_character = $opts->{indent_character} || ' ';
+    $opt_preserve_newlines = exists $opts->{preserve_newlines} ? $opts->{preserve_newlines} : 1;
+    $opt_indent_level = $opts->{indent_level} ||= 0;
 
     # -------------------------------------
     $indent_string = '';
-    while ( $indent_size-- ) {
-        $indent_string .= $indent_character;
+    while ( $opt_indent_size-- ) {
+        $indent_string .= $opt_indent_character;
     }
     @input = split('', $js_source_code);
     
@@ -63,7 +64,9 @@ sub js_beautify {
         if ( $token_type eq 'TK_START_EXPR' ) {
             $var_line = 0;
             set_mode('EXPRESSION');
-            if ( $last_type eq 'TK_END_EXPR' || $last_type eq 'TK_START_EXPR' ) {
+            if ( $last_text eq ';' ) {
+                print_newline();
+            } elsif ( $last_type eq 'TK_END_EXPR' || $last_type eq 'TK_START_EXPR' ) {
                 # do nothing on (( and )( and ][ and ]( ..
             } elsif ( $last_type ne 'TK_WORD' && $last_type ne 'TK_OPERATOR' ) {
                 print_space();
@@ -209,6 +212,10 @@ sub js_beautify {
                     $var_line = 0;
                 }
             }
+            if ($var_line && $token_text eq ',' && $current_mode eq 'EXPRESSION') {
+                # do not break on comma, for(var a = 1, b = 2)
+                $var_line_tainted = 0;
+            }
             if ( $token_text eq ':' && $in_case ) {
                 print_token(); # colon really asks for separate treatment
                 print_newline();
@@ -325,7 +332,7 @@ sub print_newline {
     if ( $output[ scalar @output - 1 ] ne "\n" || ! $ignore_repeated ) {
         push @output, "\n";
     }
-    foreach my $i ( 0 .. $indent_level - 1 ) {
+    foreach my $i ( 0 .. $opt_indent_level - 1 ) {
         push @output, $indent_string;
     }
 }
@@ -341,11 +348,11 @@ sub print_token {
     push @output, $token_text;
 }
 sub indent {
-    $indent_level++;
+    $opt_indent_level++;
 }
 sub unindent {
-    if ( $indent_level ) {
-        $indent_level--;
+    if ( $opt_indent_level ) {
+        $opt_indent_level--;
     }
 }
 sub remove_indent {
@@ -375,13 +382,16 @@ sub get_next_token {
             $n_newlines += 1;
         }
     } while ( grep { $_ eq $c } @whitespace );
-    if ( $n_newlines > 1 ) {
-        foreach my $i ( 0 .. 1 ) {
-            my $flag = ( $i == 0 ) ? 1 : 0;
-            print_newline( $flag );
+    my $wanted_newline = 0;
+    if ( $opt_preserve_newlines ) {
+        if ( $n_newlines > 1 ) {
+            foreach my $i ( 0 .. 1 ) {
+                my $flag = ( $i == 0 ) ? 1 : 0;
+                print_newline( $flag );
+            }
         }
+        $wanted_newline = ($n_newlines == 1) ? 1 : 0;
     }
-    my $wanted_newline = ($n_newlines == 1 ) ? 1 : 0;
     if ( grep { $c eq $_ } @wordchar ) {
         if ( $parser_pos < scalar @input ) {
             while ( grep { $input[$parser_pos] eq $_ } @wordchar ) {
@@ -515,11 +525,11 @@ JavaScript::Beautifier - Beautify Javascript (beautifier for javascript)
         indent_character => ' ',
     } );
 
-=head1 EXPORT
+=head1 DESCRIPTION
 
 This module is mostly a Perl-rewrite of L<http://github.com/einars/js-beautify/tree/master/beautify.js>
 
-You can check it through L<http://elfz.laacz.lv/beautify/>
+You can check it through L<http://jsbeautifier.org/>
 
 =head1 FUNCTIONS
 
@@ -527,13 +537,36 @@ You can check it through L<http://elfz.laacz.lv/beautify/>
 
 beautify javascript.
 
-$opts is a HASHREF, which contains indent_size and indent_character. if you prefer Tab than Space, try:
+=head3 options
+
+=over 4
+
+=item indent_size
+
+=item indent_character
+
+if you prefer Tab than Space, try:
 
     {
         indent_size => 1,
         indent_character => "\t",
     }
 
+=item preserve_newlines
+
+default is 1.
+
+    my $in = "var\na=dont_preserve_newlines";
+    my $out = "var a = dont_preserve_newlines";
+    my $js = js_beautify( $in, { preserve_newlines => 0 } );
+    # $out eq $js
+    $in = "var\na=do_preserve_newlines";
+    $out = "var\na = do_preserve_newlines";
+    $js = js_beautify( $in, { preserve_newlines => 1 } );
+    # $out eq $js
+
+=back
+
 =head1 AUTHOR
 
 Fayland Lam, C<< <fayland at gmail.com> >>
